#!/usr/bin/env python
# -*- coding: utf-8 -*-

#########################################################################
#    Copyright (C) 2010 Sergio Villar Senin <svillar@igalia.com>
#
#    This file is part of ReSiStance
#
#    ReSiStance is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ReSiStance is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
#########################################################################

from pango import ELLIPSIZE_END
from portrait import FremantleRotation
from threading import Thread
from webkit import WebView
import constants
import feedparser
import gettext
import gobject
import gtk
import hildon
import os
import osso
import pygtk
pygtk.require('2.0')
# For unescape()
import re, htmlentitydefs
import time
import urllib2
import urllib

_ = gettext.gettext

# Pre-cache colors
color_style = gtk.rc_get_style_by_paths(gtk.settings_get_default() , 'GtkButton', 'osso-logical-colors', gtk.Button)
active_color = color_style.lookup_color(constants.ACTIVE_TEXT_COLOR)
default_color = color_style.lookup_color(constants.DEFAULT_TEXT_COLOR)
secondary_color = color_style.lookup_color(constants.SECONDARY_TEXT_COLOR)
del color_style

# From http://effbot.org/zone/re-sub.htm#unescape-html
def unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

def get_author_from_item(item):
    if 'publisher_detail' in item:
        author = item.publisher_detail.name
    elif 'author' in item:
        author = item.author
    else:
        words = item.title.rsplit(":")
        if len(words) > 1:
            author = words[0].lstrip()
        else:
            author = ''

    return author

class FeedsView(hildon.GtkTreeView):

    FEEDS_ICON_COLUMN = 0
    FEEDS_NAME_COLUMN = 1
    FEEDS_DATA_COLUMN = 2
    FEEDS_READ_COLUMN = 3

    DUMMY_FEED_STATUS = -1

    def __init__(self, ui_mode):
        super(FeedsView, self).__init__(ui_mode)

        text_renderer = gtk.CellRendererText()
        text_renderer.set_property('ellipsize', ELLIPSIZE_END)

        pix_renderer = gtk.CellRendererPixbuf()

        # Add columns
        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = self.FEEDS_ICON_COLUMN)
        self.append_column(column);

        column = gtk.TreeViewColumn('Name', text_renderer, markup = self.FEEDS_NAME_COLUMN)
        column.set_expand(True)
        self.append_column(column);

        date_renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Date', date_renderer, markup = self.FEEDS_READ_COLUMN)
        self.append_column(column);

    def get_feed_title_markup(self, feed_data):
        if 'subtitle' in feed_data.feed:
            secondary = feed_data.feed.subtitle
        else:
            if 'link' in  feed_data.feed:
                secondary = feed_data.feed.link
            else:
                secondary = feed_data.href

        return '%s\n<span face="monospace" foreground="%s"' \
            'size="x-small">%s</span>' % \
            (feed_data.feed.title, secondary_color.to_string(), secondary)

class SettingsDialog(gtk.Dialog):

    def __init__(self, parent, settings):
        super(SettingsDialog, self).__init__(_('Settings'), parent,
                                                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                                (gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))

        self.settings = settings

        # Orientation picker
        self.selector_orientation = hildon.TouchSelector(text=True)
        for caption in FremantleRotation.MODE_CAPTIONS:
            self.selector_orientation.append_text(caption)
        self.picker_orientation = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
        self.picker_orientation.set_selector(self.selector_orientation)
        self.picker_orientation.set_alignment(0, 0.5, 0, 0)
        self.picker_orientation.set_title(_("Screen orientation"))
        self.vbox.add(self.picker_orientation)
        self.picker_orientation.show()

        # Font size picker
        self.selector_font_size = hildon.TouchSelector(text=True)
        for i in constants.font_size_range:
            self.selector_font_size.append_text(str(i))
        self.picker_font_size = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
        self.picker_font_size.set_selector(self.selector_font_size)
        self.picker_font_size.set_alignment(0, 0.5, 0, 0)
        self.picker_font_size.set_title(_("Default font size"))
        self.vbox.add(self.picker_font_size)
        self.picker_font_size.show()

        # Load images check button
        self.auto_load_images = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.auto_load_images.set_label(_('Auto load images'))
        self.vbox.add(self.auto_load_images)
        self.auto_load_images.show()

        # Automatic download check button
        self.automatic_download = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.automatic_download.set_label(_('Automatically download Podcasts'))
        self.automatic_download.connect('clicked', self._auto_download_cb)
        self.vbox.add(self.automatic_download)
        self.automatic_download.show()

        self.picker_download_folder = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
        self.picker_download_folder.set_title(_('Auto Download Folder'))
        self.picker_download_folder.set_value(self.settings.auto_download_folder)
        self.picker_download_folder.set_alignment(0, 0.5, 0, 0)
        self.picker_download_folder.connect('clicked', self._download_folder_cb)
        self.picker_download_folder.set_style(hildon.BUTTON_STYLE_PICKER)
        self.picker_download_folder.set_sensitive(self.settings.auto_download)
        self.vbox.add(self.picker_download_folder)
        self.picker_download_folder.show()

        # Load settings
        self.auto_load_images.set_active(self.settings.auto_load_images)
        self.automatic_download.set_active(self.settings.auto_download)
        self.selector_orientation.set_active(0, self.settings.rotation_mode)
        try:
            font_size_index = constants.font_size_range.index(self.settings.default_font_size)
        except ValueError:
            # defaults to 16pt
            font_size_index = constants.font_size_range.index(16)
        self.selector_font_size.set_active(0, font_size_index)

    def _auto_download_cb(self, button):
        self.picker_download_folder.set_sensitive(button.get_active())

    def _download_folder_cb(self, button):
        stack = hildon.WindowStack.get_default()
        window = stack.peek()

        chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        chooser.set_transient_for(window)
        chooser.set_title(_('Select a folder for automatic downloads'))
        chooser.set_current_folder(constants.MYDOCS_DIR)

        chooser.set_default_response(gtk.RESPONSE_OK)

        response = chooser.run()
        folder = chooser.get_filename() + '/'
        chooser.destroy()

        self.settings.auto_download_folder = folder
        button.set_value(folder)

class NewFeedDialog(gtk.Dialog):

    def __init__(self, parent):
        super(NewFeedDialog, self).__init__(_('New Feed'), parent,
                                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                            (gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))

        self.entry = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.entry.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
        self.entry.connect('changed', self._entry_changed_cb)

        caption = gtk.HBox(False, 16)
        caption.pack_start(gtk.Label(_('Address') + ':'), False, False)
        caption.pack_start(self.entry)
        self.vbox.add(caption)

        self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)

    def _entry_changed_cb(self, entry):
        if len(entry.get_text()) > 0:
            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
        else:
            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)

class FindFeedsDialog(gtk.Dialog):
    def __init__(self, parent):
        super(FindFeedsDialog, self).__init__(_('Find Feeds'), parent,
                                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                            (_('Find'), gtk.RESPONSE_ACCEPT))


        self.entry = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.entry.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
        self.entry.connect('changed', self._entry_changed_cb)

        caption = gtk.HBox(False, 16)
        caption.pack_start(gtk.Label(_('Keywords') + ':'), False, False)
        caption.pack_start(self.entry)
        self.vbox.add(caption)

        self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)

    def _entry_changed_cb(self, entry):
        if len(entry.get_text()) > 0:
            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
        else:
            self.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)

    def _delete_event_cb(self, widget, event):
        return True #stop the signal propagation

class FeedsWindow(hildon.StackableWindow):

    FEEDS_MODEL_PIXBUF_COLUMN = 0
    FEEDS_MODEL_TITLE_COLUMN = 1
    FEEDS_MODEL_DATA_COLUMN = 2
    FEEDS_MODEL_READ_COLUMN = 3
    FEEDS_MODEL_VISITS_COLUMN = 4

    def __init__(self, manager, settings, conn_manager):
        super(FeedsWindow, self).__init__()

        self.manager = manager
        self.settings = settings
        self._conn_manager = conn_manager
        self._conn_manager.connect('connection-changed', self._on_connection_changed)

        # Feeds
        self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, int))
        self.view.connect ("row-activated", self._on_feed_activated)
        self.view.show()

        # Used for osso context in rotation object
        app_name = constants.RSS_NAME
        app_version = constants.RSS_VERSION

        # Auto rotate
        self.rotation_object = FremantleRotation(app_name, self, app_version,
                                                 FremantleRotation.AUTOMATIC)

        # Edit toolbar
        self.edit_toolbar_button_handler = 0
        self.edit_toolbar = hildon.EditToolbar()
        self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)

        self.align = gtk.Alignment(0, 0, 1, 1)
        self.align.set_padding(4, 0 , 16, 16)

        pannable = hildon.PannableArea()
        pannable.add (self.view)
        pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
        pannable.show()

        self.align.add(pannable)
        self.add(self.align)
        self.align.show()

        self._create_menu()
        self.set_title(app_name)
        self.connect('configure-event', self._on_configuration_changed)
        self.connect('delete-event', self._on_exit)
        hildon.Program.get_instance().add_window(self)

        # Apply settings
        self.settings.load()
        self._sort(self.settings.feeds_order)
        if self.settings.feeds_order == constants.DESCENDING_ORDER:
            self.descending_filter_button.set_active(True)
        elif self.settings.feeds_order == constants.VISITS_ORDER:
            self.visits_filter_button.set_active(True)

        # Force online/offline status evaluation
        self._on_connection_changed(self._conn_manager)

        # Load Feeds (could be none)
        try:
            self.manager.load(self._feed_data_loaded)
        except IOError:
            self._show_initial_dialog()

    def _on_connection_changed(self, conn_manager):
        if self._conn_manager.is_online():
            self._new_feed_button.show()
            self._update_all_button.show()
            self._find_feeds_button.show()
            self._import_feeds_button.show()
        else:
            self._new_feed_button.hide()
            self._update_all_button.hide()
            self._find_feeds_button.hide()
            self._import_feeds_button.hide()

    def _sort(self, order):
        if (order == constants.ASCENDING_ORDER):
            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_TITLE_COLUMN,
                                                     gtk.SORT_ASCENDING)
        elif (order == constants.DESCENDING_ORDER):
            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_TITLE_COLUMN,
                                                     gtk.SORT_DESCENDING)
        else:
            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_VISITS_COLUMN,
                                                     gtk.SORT_DESCENDING)

    def _add_dummy_feed(self, url):

        store = self.view.get_model()
        feed_iter = store.append()
        dummy_feed_data = feedparser.FeedParserDict()
        dummy_feed_data.status = self.view.DUMMY_FEED_STATUS
        dummy_feed_data.href = url
        dummy_feed_data.feed = feedparser.FeedParserDict()
        dummy_feed_data.feed.title = _('Retrieving info...')
        store.set(feed_iter,
                  self.FEEDS_MODEL_PIXBUF_COLUMN, None,
                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(dummy_feed_data),
                  self.FEEDS_MODEL_DATA_COLUMN, dummy_feed_data,
                  self.FEEDS_MODEL_READ_COLUMN, '',
                  self.FEEDS_MODEL_VISITS_COLUMN, 0)

        return (dummy_feed_data,feed_iter)

    def _on_feed_activated(self, treeview, path, column):
        feed_iter = self.view.get_model().get_iter(path)
        feed_data = self.view.get_model().get_value(feed_iter, self.FEEDS_MODEL_DATA_COLUMN)

        # Check that we're not trying to open a dummy feed
        if feed_data.status == self.view.DUMMY_FEED_STATUS:
            hildon.hildon_banner_show_information(self, '', _('Wait until feed is refreshed'))
            return

        news_window = EntriesWindow(feed_data, self.manager, self.settings, self._conn_manager)
        hildon.Program.get_instance().add_window(news_window)
        news_window.show()

        # Update the visits count
        feed_data.visits += 1
        self.view.get_model().set_value(feed_iter, self.FEEDS_MODEL_VISITS_COLUMN, feed_data.visits)

        # When the news window is destroyed reset the unread cache
        news_window.connect('delete-event', self._on_news_window_destroyed,
                            gtk.TreeRowReference(self.view.get_model(), path))

    def _on_news_window_destroyed(self, news_window, event, row_reference):
        feed_iter = self.view.get_model().get_iter(row_reference.get_path())
        feed_data = self.view.get_model().get_value(feed_iter, self.FEEDS_MODEL_DATA_COLUMN)

        # Reset unread cache (the value have most likely changed).
        unread_count = len([entry for entry in feed_data.entries if entry.read == False])
        self.view.get_model().set(feed_iter, self.FEEDS_MODEL_READ_COLUMN,
                                  self._get_visual_unread_text(unread_count))

    def _on_configuration_changed(self, window, event):
        # Change alignment padding. We don't want padding in portrait
        # Need to save space
        if event.width > event.height:
            self.align.set_padding(4, 0 , 16, 16)
        else:
            self.align.set_padding(4, 0 , 4, 4)

    def _on_exit(self, window, event):
        # Save the feeds status
        self.hide_on_delete()
        self.manager.save(self._on_feed_data_saved)
        self.settings.save()
        return True

    def _on_feed_data_saved(self, user_data):
        # Close application
        gtk.main_quit()

    def _create_menu(self):
        menu = hildon.AppMenu()

        # Sorting filter
        self.ascending_filter_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.ascending_filter_button.set_mode(False)
        self.ascending_filter_button.set_label(_('A-Z'))
        self.ascending_filter_button.connect('clicked', self._sort_ascending_cb)
        menu.add_filter(self.ascending_filter_button)
        self.descending_filter_button = \
            hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                  group = self.ascending_filter_button)
        self.descending_filter_button.set_mode(False)
        self.descending_filter_button.set_label(_('Z-A'))
        self.descending_filter_button.connect('clicked', self._sort_descending_cb)
        menu.add_filter(self.descending_filter_button)
        self.visits_filter_button = \
            hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                  group = self.ascending_filter_button)
        self.visits_filter_button.set_mode(False)
        self.visits_filter_button.set_label(_('Favorites'))
        self.visits_filter_button.connect('clicked', self._sort_visits_cb)
        menu.add_filter(self.visits_filter_button)

        self._new_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._new_feed_button.set_label(_('New Feed'))
        self._new_feed_button.connect('clicked', self._new_feed_cb)
        menu.append(self._new_feed_button)

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Remove Feed'))
        button.connect('clicked', self._remove_feed_cb)
        menu.append(button)

        self._update_all_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._update_all_button.set_label(_('Update all'))
        self._update_all_button.connect('clicked', self._update_all_cb)
        menu.append(self._update_all_button)

        self._import_feeds_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._import_feeds_button.set_label(_('Import Feeds'))
        self._import_feeds_button.connect('clicked', self._import_feed_cb)
        menu.append(self._import_feeds_button)

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Export Feeds'))
        button.connect('clicked', self._export_feed_cb)
        menu.append(button)
        self.export_opml_button = button

        self._find_feeds_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._find_feeds_button.set_label(_('Find Feeds'))
        self._find_feeds_button.connect('clicked', self._find_feed_cb)
        menu.append(self._find_feeds_button)

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Settings'))
        button.connect('clicked', self._preferences_cb)
        menu.append(button)

        menu.show_all()
        self.set_app_menu(menu)

    def _new_feed_cb(self, button):
        dialog = NewFeedDialog(self)
        dialog.connect('response', self._new_feed_response_cb)
        dialog.show_all()

    def _remove_feed_cb(self, button):

        # tree view edit mode
        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_EDIT)
        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.view.get_selection().unselect_all()

        self.edit_toolbar.set_label(_('Select Feeds to remove'))
        self.edit_toolbar.set_button_label(_('Remove'))
        if self.edit_toolbar.handler_is_connected (self.edit_toolbar_button_handler):
            self.edit_toolbar.disconnect(self.edit_toolbar_button_handler)
        self.edit_toolbar_button_handler = \
            self.edit_toolbar.connect('button-clicked', self._remove_button_clicked_cb)

        self.set_edit_toolbar(self.edit_toolbar)
        self.edit_toolbar.show()
        self.fullscreen()

    def _remove_button_clicked_cb(self, button):
        selection = self.view.get_selection()
        selected_rows = selection.get_selected_rows()
        model, paths = selected_rows
        if not paths:
            hildon.hildon_banner_show_information(self, '', _('Please select at least one Feed'))
            return

        removed = []
        for path in paths:
            try:
                self.manager.remove_feed(model[path][FeedsView.FEEDS_DATA_COLUMN])
                feed_data_list = self.manager.get_feed_list()
                if len(feed_data_list) == 0:
                    self.export_opml_button.hide()
            except IOError:
                print 'Could not remove', constants.RSS_DB_FILE
                hildon.hildon_banner_show_information(self, '', _('File error while removing Feed'))
            else:
                removed.append(gtk.TreeRowReference(model, path))

        for reference in removed:
            model.remove(model.get_iter(reference.get_path()))

        # restore normal mode
        self._restore_normal_mode(button)

    def _restore_normal_mode(self, button):

        # tree view normal mode
        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_NORMAL)
        self.view.get_selection().unselect_all()

        self.edit_toolbar.hide()
        self.unfullscreen()

    def _preferences_cb(self, button):
        dialog = SettingsDialog(self, self.settings)
        dialog.connect('response', self._preferences_response_cb)
        dialog.show_all()

    def _update_all_cb(self, button):
        # Quickly show feedback to user
        hildon.hildon_gtk_window_set_progress_indicator(self, True)

        # Ask manager to update feed
        self.manager.update_all(self._all_feeds_updated_cb, None)

    def _get_visual_unread_text(self, unread_count):
        if unread_count:
            return '<span size="xx-small">%d ' \
                '%s\n%s</span>' % (unread_count, _('unread'), _('items'))
        else:
            return ''

    def _all_feeds_updated_cb(self, data):
        hildon.hildon_gtk_window_set_progress_indicator(self, False)

        # TODO: Update counters
        store = self.view.get_model()
        for row in store:
            unread_count = len([entry for entry in row[self.FEEDS_MODEL_DATA_COLUMN].entries if entry.read == False])
            row[self.FEEDS_MODEL_READ_COLUMN] = self._get_visual_unread_text(unread_count)

    def _new_feed_response_cb(self, dialog, response):

        if response == gtk.RESPONSE_ACCEPT:

            url = dialog.entry.get_text()

            # Quickly show feedback to user
            hildon.hildon_gtk_window_set_progress_indicator(self, True)

            # Insert a dummy row while information is retrieved
            dummy_feed_data, feed_iter = self._add_dummy_feed(url)
            path = self.view.get_model().get_path(feed_iter)
            row_reference = gtk.TreeRowReference(self.view.get_model(), path)

            # Add feed to manager
            self.manager.add_feed(url, self._feed_added_cb, row_reference)

            dialog.destroy()

    def _feed_added_cb(self, pixbuf, new_feed_data, row_reference=None):

        feed_data_list = self.manager.get_feed_list()
        if len(feed_data_list) > 0:
            self.export_opml_button.show()

        # Remove progress information
        hildon.hildon_gtk_window_set_progress_indicator(self, False)

        store = self.view.get_model()
        if row_reference != None:
            feed_iter = store.get_iter(row_reference.get_path())
        else:
            feed_iter = store.append()

        # Check that the feed data was retrieved
        if new_feed_data == None:
            message = _('Could not retrieve feed')
            dummy_data = store.get_value (feed_iter, self.FEEDS_MODEL_DATA_COLUMN)
            if dummy_data:
                message += 'from ' + dummy_data.href
            store.remove(feed_iter)
            hildon.hildon_banner_show_information(self, '', message)
            return

        # Update model
        store.set(feed_iter,
                  self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(new_feed_data),
                  self.FEEDS_MODEL_DATA_COLUMN, new_feed_data,
                  self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(len(new_feed_data.entries)),
                  self.FEEDS_MODEL_VISITS_COLUMN, new_feed_data['visits'])

        # Take screenshot
        hildon.hildon_gtk_window_take_screenshot(self, False)
        hildon.hildon_gtk_window_take_screenshot(self, True)

    def _preferences_response_cb(self, dialog, response):

        if response == gtk.RESPONSE_ACCEPT:
            self.settings.auto_load_images = dialog.auto_load_images.get_active()
            self.settings.rotation_mode = dialog.selector_orientation.get_active(0)
            self.settings.default_font_size = int(dialog.selector_font_size.get_current_text())
            self.settings.auto_download = dialog.automatic_download.get_active()

            dialog.destroy()

            # Update rotation
            self.rotation_object.set_mode(self.settings.rotation_mode)

    def _feed_exported_cb(self, exported):
        if exported:
            message = _('Feeds exported')
        else:
            message = _('Error exporting feeds')
        hildon.hildon_banner_show_information(self, '', message)
        # Remove progress information
        hildon.hildon_gtk_window_set_progress_indicator(self, False)

    def _initial_dialog_new_cb(self, button, dialog):
        dialog.hide()
        self._new_feed_cb(button)
        dialog.destroy()

    def _initial_dialog_import_cb(self, button, dialog):
        dialog.hide()
        self._import_feed_cb(button)
        dialog.destroy()

    def _initial_dialog_find_cb(self, button, dialog):
        dialog.hide()
        self._find_feed_cb(button)
        dialog.destroy()

    def _show_initial_dialog(self):
        dialog = gtk.Dialog()
        dialog.set_title(_('Start adding feeds'))

        # Create buttons
        add_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        add_feed_button.set_label(_('New feed'))
        add_feed_button.connect('clicked', self._initial_dialog_new_cb, dialog)

        import_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        import_feed_button.set_label(_('Import Feeds'))
        import_feed_button.connect('clicked', self._initial_dialog_import_cb, dialog)

        find_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        find_feed_button.set_label(_('Find Feeds'))
        find_feed_button.connect('clicked', self._initial_dialog_find_cb, dialog)

        dialog.vbox.add(add_feed_button)
        dialog.vbox.add(import_feed_button)
        dialog.vbox.add(find_feed_button)

        add_feed_button.show()
        import_feed_button.show()
        find_feed_button.show()
        dialog.show()

    def _feed_data_loaded(self, user_data):
        # TODO: migrate to "with" statement when available
        try:
            feed_data_list = self.manager.get_feed_list()
            if len(feed_data_list) == 0:
                self.export_opml_button.hide()
        except IOError:
            print 'cannot open', constants.RSS_DB_FILE
        else:
            store = self.view.get_model()

            for feed_data in feed_data_list:
                feed_iter = store.append()
                unread_count = len([entry for entry in feed_data.entries if entry.read == False])
                store.set(feed_iter,
                          self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(feed_data),
                          self.FEEDS_MODEL_DATA_COLUMN, feed_data,
                          self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(unread_count),
                          self.FEEDS_MODEL_VISITS_COLUMN, feed_data['visits'])
                # Favicon might require a network download
                path = self.view.get_model().get_path(feed_iter)
                row_reference = gtk.TreeRowReference(store, path)
                pixbuf = self.manager.get_favicon(feed_data.feed.link or feed_data.href,
                                             self._get_favicon_cb, row_reference)

        # If there is no data then show a dialog
        if len(feed_data_list) == 0:
            self._show_initial_dialog()

    def _get_favicon_cb(self, pixbuf, row_reference):
        if pixbuf == None:
            return

        store = self.view.get_model()
        feed_iter = store.get_iter(row_reference.get_path())
        store.set(feed_iter, self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf)

    def _find_feed_response_cb(self, dialog, response):
        if response == gtk.RESPONSE_ACCEPT:
            key_words = dialog.entry.get_text()

            # Quickly show feedback to user
            hildon.hildon_gtk_window_set_progress_indicator(dialog, True)

            self.manager.find_feed(key_words, dialog, self._found_cb)
            dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)

            # Disable canceling
            dialog.connect('delete-event', lambda widget,event : True)

    def _find_window_urls_found_cb(self, find_window, urls):

        for url in urls:
            # Quickly show feedback to user
            hildon.hildon_gtk_window_set_progress_indicator(self, True)

            # Insert a dummy row while information is retrieved
            dummy_feed_data, feed_iter = self._add_dummy_feed(url)
            path = self.view.get_model().get_path(feed_iter)
            row_reference = gtk.TreeRowReference(self.view.get_model(), path)

            # Add feed to manager
            self.manager.add_feed(url, self._feed_added_cb, row_reference)

    def _found_cb(self, key_words, dialog, feeds_info):
        if feeds_info == None:
            # Something went wrong
            hildon.hildon_banner_show_information(self, '', _('Error finding feeds. Server error.'))
            dialog.destroy()
            return

        news_window = FindWindow(feeds_info, key_words, self)
        hildon.Program.get_instance().add_window(news_window)
        news_window.show()

        news_window.connect('urls-found', self._find_window_urls_found_cb)

        dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
        hildon.hildon_gtk_window_set_progress_indicator(dialog, False)

        dialog.destroy()

    def _sort_ascending_cb(self, button):
        self.settings.feeds_order = constants.ASCENDING_ORDER
        self._sort(constants.ASCENDING_ORDER)

    def _sort_descending_cb(self, button):
        self.settings.feeds_order = constants.DESCENDING_ORDER
        self._sort(constants.DESCENDING_ORDER)

    def _sort_visits_cb(self, button):
        self.settings.feeds_order = constants.VISITS_ORDER
        self._sort(constants.VISITS_ORDER)

    def _export_feed_cb(self, button):
        stack = hildon.WindowStack.get_default()
        window = stack.peek()

        chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SAVE)
        chooser.set_transient_for(window)
        # Don't like this at all. Isn't there any define in the platform ?
        chooser.set_current_folder(constants.MYDOCS_DIR)
        chooser.set_current_name('resistance-feeds')

        #Overwrite the file if already exists
        chooser.set_do_overwrite_confirmation(True)
        chooser.set_default_response(gtk.RESPONSE_OK)

        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            hildon.hildon_gtk_window_set_progress_indicator(self, True)
            self.manager.export_opml(chooser.get_filename() + '.opml', self._feed_exported_cb)
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no files selected'
        chooser.destroy()

    def _import_feed_cb(self, button):
        stack = hildon.WindowStack.get_default()
        window = stack.peek()

        #Calling a file chooser
        chooser = hildon.FileChooserDialog(window ,gtk.FILE_CHOOSER_ACTION_OPEN)
        chooser.set_transient_for(window)
        # Don't like this at all. Isn't there any define in the platform ?
        chooser.set_current_folder(constants.MYDOCS_DIR)
        chooser.set_default_response(gtk.RESPONSE_OK)

        #Filter for opml files
        filter = gtk.FileFilter()
        filter.set_name("OPML")
        filter.add_pattern("*.opml")
        chooser.add_filter(filter)

        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            hildon.hildon_gtk_window_set_progress_indicator(self, True)
            self.manager.import_opml(chooser.get_filename(), self._import_opml_cb)
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no files selected'

        chooser.destroy()

    def _import_opml_cb(self, feed_url_list, data):
        if len(feed_url_list) == 0:
            return

        hildon.hildon_gtk_window_set_progress_indicator(self, True)

        for feed_url in feed_url_list:
            # Insert a dummy row while information is retrieved
            dummy_feed_data, feed_iter = self._add_dummy_feed(feed_url)
            path = self.view.get_model().get_path(feed_iter)
            row_reference = gtk.TreeRowReference(self.view.get_model(), path)

            # Add feed to manager
            self.manager.add_feed(feed_url, self._feed_added_cb, row_reference)

    def _find_feed_cb(self, button):
        dialog = FindFeedsDialog(self)
        dialog.connect('response', self._find_feed_response_cb)
        dialog.show_all()

class EntriesView(hildon.GtkTreeView):

    ENTRIES_ICON_COLUMN = 0
    ENTRIES_NAME_COLUMN = 1
    ENTRIES_DATA_COLUMN = 2
    ENTRIES_DATE_COLUMN = 3

    def __init__(self, feed_title, ui_mode):
        super(EntriesView, self).__init__(ui_mode)

        self.fallback_author = feed_title

        # Add columns
        pix_renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = self.ENTRIES_ICON_COLUMN)
        self.append_column(column);

        text_renderer = gtk.CellRendererText()
        text_renderer.set_property('ellipsize', ELLIPSIZE_END)
        column = gtk.TreeViewColumn('Name', text_renderer, markup = self.ENTRIES_NAME_COLUMN)
        column.set_expand(True)
        self.append_column(column);

        date_renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Date', date_renderer, markup = self.ENTRIES_DATE_COLUMN)
        column.set_expand(False)
        self.append_column(column);

    def get_visual_entry_text(self, entry):

        # TODO: show entry.summary ?
        author = get_author_from_item(entry)
        if author == '':
            author = self.fallback_author

        words = entry.title.rsplit(":")
        if len(words) > 1 and author == words[0].lstrip():
            title = words[1].lstrip()
        else:
            title = entry.title

        if 'read' in entry and entry['read'] == True:
            color = secondary_color.to_string()
        else:
            color = active_color.to_string()

        return '<span foreground="' + color + '">' \
            + gobject.markup_escape_text(unescape(author)) \
            + '</span>\n<span size="small">' \
            + gobject.markup_escape_text(unescape(title)) + '</span>'

    def get_visual_entry_date(self, entry):
        if entry.has_key('updated_parsed') == False:
            if entry.has_key('updated'):
                return '<span size="xx-small">%s</span>' % \
                    entry.updated
            else:
                return ''

        now = time.localtime()

        # Today
        if now.tm_yday == entry.updated_parsed.tm_yday:
            return '<span size="xx-small">%s</span>' % \
                time.strftime('%X', entry.updated_parsed)
        # This week
        elif now.tm_yday - entry.updated_parsed.tm_yday < 7 and \
                entry.updated_parsed.tm_wday < now.tm_wday:
            return '<span size="xx-small">%s</span>' % \
                time.strftime('%A', entry.updated_parsed)
        # This year
        elif now.tm_year == entry.updated_parsed.tm_year:
            return '<span size="xx-small">%s</span>' % \
                time.strftime('%d %B', entry.updated_parsed)
        # Anything else
        else:
            return '<span size="xx-small">%s</span>' % \
                time.strftime('%x', entry.updated_parsed)

class EntriesWindow(hildon.StackableWindow):

    ENTRIES_MODEL_TEXT_COLUMN = 1
    ENTRIES_MODEL_DATA_COLUMN = 2
    ENTRIES_MODEL_DATE_COLUMN = 3

    def __init__(self, feed_data, manager, settings, conn_manager):
        super(EntriesWindow, self).__init__()

        self.manager = manager
        self.settings = settings
        self.feed_data = feed_data
        self._conn_manager = conn_manager
        self._conn_manager.connect('connection-changed', self._on_connection_changed)

        self.set_title(unescape(self.feed_data.feed.title))
        self.toolbar = gtk.Toolbar()

        # Edit toolbar
        self.edit_toolbar_button_handler = 0
        self.edit_toolbar = hildon.EditToolbar()
        self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)

        # Feeds
        self.view = EntriesView(feed_data.feed.title, gtk.HILDON_UI_MODE_NORMAL)
        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str))
        self.view.connect ("row-activated", self._on_entry_activated)
        self.view.show()

        model = self.view.get_model()

        for entry in self.feed_data.entries:
            entry_iter = model.append()
            model.set(entry_iter,
                      self.ENTRIES_MODEL_TEXT_COLUMN, self.view.get_visual_entry_text(entry),
                      self.ENTRIES_MODEL_DATA_COLUMN, entry,
                      self.ENTRIES_MODEL_DATE_COLUMN, self.view.get_visual_entry_date(entry))

        self.align = gtk.Alignment(0, 0, 1, 1)
        self.align.set_padding(4, 0 , 16, 16)

        pannable = hildon.PannableArea()
        pannable.add (self.view)
        pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
        pannable.show()

        self.align.add(pannable)
        self.add(self.align)
        self.align.show()

        # Update feed button
        action_box = hildon.GtkTreeView.get_action_area_box(self.view)
        self._update_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._update_feed_button.set_label(_('Update feed'))
        self._update_feed_button.connect('clicked', self._update_feed_cb)

        action_box.pack_start(self._update_feed_button, True, True)

        self._create_menu()

        if 'itunes' not in self.feed_data.namespaces:
            self._download_all_button.hide()

        self.connect('configure-event', self._on_configuration_changed)

        # Force online/offline status evaluation
        self._on_connection_changed(self._conn_manager)

    def _on_connection_changed(self, conn_manager):
        if conn_manager.is_online():
            self._update_feed_button.show()
            # Check that we should not show it
            if 'itunes' in self.feed_data.namespaces:
                self._download_all_button.show()
            hildon.GtkTreeView.set_action_area_visible(self.view, True)
        else:
            self._update_feed_button.hide()
            self._download_all_button.hide()
            hildon.GtkTreeView.set_action_area_visible(self.view, False)

    def _create_menu(self):
        menu = hildon.AppMenu()

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Mark Entries as Read'))
        button.connect('clicked', self._mark_read_cb, True)
        menu.append(button)

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Mark Entries as Not Read'))
        button.connect('clicked', self._mark_read_cb, False)
        menu.append(button)

        self._download_all_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._download_all_button.set_label(_('Download all'))
        self._download_all_button.connect('clicked', self._download_all_items_cb)
        menu.append(self._download_all_button)

        menu.show_all()
        self.set_app_menu(menu)

    def _mark_read_cb(self, button, read):
        # tree view edit mode
        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_EDIT)
        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.view.get_selection().unselect_all()

        if read:
            self.edit_toolbar.set_label(_('Select Feeds to mark as Read'))
        else:
            self.edit_toolbar.set_label(_('Select Feeds to mark as Not Read'))
        self.edit_toolbar.set_button_label(_('Mark'))
        if self.edit_toolbar.handler_is_connected (self.edit_toolbar_button_handler):
            self.edit_toolbar.disconnect(self.edit_toolbar_button_handler)
        self.edit_toolbar_button_handler = \
            self.edit_toolbar.connect('button-clicked', self._mark_read_button_clicked_cb, read)

        self.set_edit_toolbar(self.edit_toolbar)
        self.edit_toolbar.show()
        self.fullscreen()

    def _mark_read_button_clicked_cb(self, button, read):
        selection = self.view.get_selection()
        selected_rows = selection.get_selected_rows()
        model, paths = selected_rows
        if not paths:
            hildon.hildon_banner_show_information(self, '', _('Please select at least one Entry'))
            return

        for path in paths:
            model[path][2]['read'] = read
            model[path][1] = self.view.get_visual_entry_text(model[path][2])

        # restore normal mode
        self._restore_normal_mode(button)

    def _restore_normal_mode(self, button):

        # tree view normal mode
        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_NORMAL)
        self.view.get_selection().unselect_all()

        self.edit_toolbar.hide()
        self.unfullscreen()

    def _on_entry_activated(self, treeview, path, column):
        item_window = ItemWindow(self.feed_data, self.manager, self.settings, self._conn_manager)
        hildon.Program.get_instance().add_window(item_window)

        item_window.connect('item-read', self._item_read_cb)
        item_window.set_item_from_model(treeview.get_model(), path)
        item_window.show()

    def _item_read_cb(self, item_window, path):
        entry = self.view.get_model()[path][self.ENTRIES_MODEL_DATA_COLUMN]

        # Set item as read and update the UI
        entry['read'] = True
        self.view.get_model()[path][self.ENTRIES_MODEL_TEXT_COLUMN] = \
            self.view.get_visual_entry_text(entry)

    def _update_feed_cb(self, button):
        url = self.feed_data.href

        # Quickly show feedback to user
        hildon.hildon_gtk_window_set_progress_indicator(self, True)
        button.set_sensitive(False)

        # Ask manager to update feed
        self.manager.update_feed(self.feed_data, self._feed_updated_cb, None)

    def _feed_updated_cb(self, data):
        hildon.hildon_gtk_window_set_progress_indicator(self, False)
        self._update_feed_button.set_sensitive(True)

        # No need to update self.feed_data as it is automatically updated.
        # The same happens with the reference stored in the FeedsWindow :-)
        model = self.view.get_model()

        model.clear()
        for entry in self.feed_data.entries:
            entry_iter = model.append()
            model.set(entry_iter,
                      self.ENTRIES_MODEL_TEXT_COLUMN, self.view.get_visual_entry_text(entry),
                      self.ENTRIES_MODEL_DATA_COLUMN, entry,
                      self.ENTRIES_MODEL_DATE_COLUMN, self.view.get_visual_entry_date(entry))

        return

    def _download_all_items_cb(self, button):
        folder = self.settings.auto_download_folder

        urls = []
        paths_files = []
        for entry in self.feed_data.entries:
            try:
                url = entry['enclosures'][0]['href']
                urls.append(url)
                paths_files.append(folder + os.path.basename(urllib.url2pathname(url)))
            except:
                pass

        hildon.hildon_gtk_window_set_progress_indicator(self, True)

        self.manager.download_all_items(urls, paths_files, self._all_items_downloaded_cb)

    def _all_items_downloaded_cb(self, downloaded):
        if downloaded:
            message = _('Items downloaded')
        else:
            message = _('Error downloading items')
        hildon.hildon_banner_show_information(self, '', message)
        # Remove progress information
        hildon.hildon_gtk_window_set_progress_indicator(self, False)

    def _on_configuration_changed(self, window, event):
        # Change alignment padding. We don't want padding in portrait
        # Need to save space
        if event.width > event.height:
            self.align.set_padding(4, 0 , 16, 16)
        else:
            self.align.set_padding(4, 0 , 4, 4)

class ItemWindowHeader(gtk.HBox):

    def __init__(self, homogeneous, spacing):
        super(ItemWindowHeader, self).__init__(homogeneous, spacing)

        # Header
        header_vbox = gtk.VBox()
        navigation_hbox = gtk.HBox(False, 0)

        # Author and URL
        self.title = gtk.Label('')
        self.title.set_alignment(0,0.5)
        self.title.set_ellipsize(ELLIPSIZE_END)
        header_vbox.pack_start(self.title, True, False)
        self.title.show()
        header_vbox.show()

        # Navigation buttons. Sizes are important. We set the size of the icon but
        # we let the button grow as needed
        self.button_up = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button_up_img = gtk.Image()
        button_up_img.set_from_icon_name(constants.ICON_UP_NAME,
                                              gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.button_up.set_image(button_up_img)
        self.button_down = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button_down_img = gtk.Image()
        button_down_img.set_from_icon_name(constants.ICON_DOWN_NAME,
                                                gtk.HILDON_SIZE_FINGER_HEIGHT)
        self.button_down.set_image(button_down_img)

        navigation_hbox.pack_start(self.button_up, False, False, 0)
        navigation_hbox.pack_start(self.button_down, False, False, 0)
        navigation_hbox.show_all()

        self.pack_start(header_vbox, True, True)
        self.pack_start(navigation_hbox, False, False)

        # Signals
        #self.url.connect('clicked', self._link_button_clicked)

    def _link_button_clicked(self, link_button):
        print 'Open ' + link_button.get_uri()

    def set_item(self, item):

        self.item = item

        self.title.set_markup('<span size="small">' + unescape(item.title) + '</span>')
        #self.url.set_uri(item.link)

class ItemWindow(hildon.StackableWindow):
    __gsignals__ = {
        "item-read": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
        }

    def __init__(self, feed_data, manager, settings, conn_manager):
        super(ItemWindow, self).__init__()

        self.manager = manager
        self.settings = settings
        self.feed_data = feed_data
        self._conn_manager = conn_manager
        self._conn_manager.connect('connection-changed', self._on_connection_changed)

        # Header
        self.item_window_header = ItemWindowHeader(False, 16)
        self.item_window_header.button_up.connect("clicked", self._up_button_clicked)
        self.item_window_header.button_down.connect("clicked", self._down_button_clicked)

        # HTML renderer
        self.view = WebView()
        self.view.set_full_content_zoom(True)

        # Disable text selection
        self.view.connect("motion-notify-event", lambda w, ev: True)

        # Set some settings
        wbk_settings = self.view.get_settings()
        wbk_settings.set_property('default_font_size', self.settings.default_font_size)
        wbk_settings.set_property('auto-load-images', self.settings.auto_load_images)
        wbk_settings.set_property('auto-shrink-images', True)

        # Pannable for showing content
        self.pannable = hildon.PannableArea()
        self.pannable.add(self.view)
        self.pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
        self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
        self.view.show()

        # Header Alignment
        header_align = gtk.Alignment(0, 0, 1, 1)
        header_align.set_padding(4, 0 , 16, 16)
        header_align.add(self.item_window_header)
        header_align.show()

        vbox = gtk.VBox(False, 8)
        vbox.pack_start(header_align, False, False)
        vbox.pack_start(self.pannable, True, True)
        self.item_window_header.show()
        self.pannable.show()

        self.add(vbox)
        vbox.show()

        self._create_menu()
        self._create_toolbar()

        # For portrait/landscape modes
        self.connect('configure-event', self._on_configuration_changed)

    def _on_connection_changed(self, conn_manager):
        if conn_manager.is_online():
            if 'enclosures' in self.item_window_header.item:
                self._download_button.show()
        else:
            self._download_button.hide()

    def _on_configuration_changed(self, window, event):

        # Show toolbar in portrait mode. Show header in landscape mode
        if event.width > event.height:
            self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
            self.item_window_header.show_all()
            self.toolbar.hide_all()
        else:
            self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
            self.item_window_header.button_up.hide()
            self.item_window_header.button_down.hide()
            self.toolbar.show_all()

    def _create_toolbar(self):

        self.toolbar = gtk.Toolbar()
        button_prev_img = gtk.Image()
        button_prev_img.set_from_icon_name(constants.ICON_UP_NAME,
                                           gtk.HILDON_SIZE_FINGER_HEIGHT)
        button_prev = gtk.ToolButton(button_prev_img, None)

        button_next_img = gtk.Image()
        button_next_img.set_from_icon_name(constants.ICON_DOWN_NAME,
                                           gtk.HILDON_SIZE_FINGER_HEIGHT)
        button_next = gtk.ToolButton(button_next_img, None)

        button_prev.set_expand(True)
        button_next.set_expand(True)
        button_prev.set_homogeneous(True)
        button_next.set_homogeneous(True)

        self.toolbar.insert (button_prev, -1)
        self.toolbar.insert (button_next, -1)

        button_prev.connect("clicked", self._up_button_clicked)
        button_next.connect("clicked", self._down_button_clicked)

        self.add_toolbar(self.toolbar)

    def _on_request_url(self, object, url, stream):
        if (url.lower().startswith("http")):
            request_thread = Thread(target=self._request_url_in_thread,
                                    args=(url, stream))
            request_thread.start()

    def _request_url_in_thread(self, url, stream):
        f = urllib2.urlopen(url)

        gtk.gdk.threads_enter()
        stream.write(f.read())
        stream.close()
        gtk.gdk.threads_leave()

    def _create_menu(self):
        menu = hildon.AppMenu()

        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        button.set_label(_('Item Details'))
        button.connect('clicked', self._item_details_cb)
        button.show()
        menu.append(button)

        self._download_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
        self._download_button.set_label(_('Download'))
        self._download_button.connect('clicked', self._download_item_cb)
        self._download_button.show()
        menu.append(self._download_button)

        menu.show_all()
        self.set_app_menu(menu)

    def _add_to_table (self, table, color, key, value, y_coord):
        label = gtk.Label('')
        label.set_markup('<span foreground="%s">%s:</span>' % (color, key))
        label.set_alignment(1, 0)
        table.attach(label, 0, 1, y_coord, y_coord + 1)

        value_label = gtk.Label('')
        value_label.set_markup(value)
        value_label.set_alignment(0, 0)
        table.attach(gtk.Label(value), 1, 2, y_coord, y_coord + 1)

    def _item_details_cb(self, button):
        dialog = gtk.Dialog()
        dialog.set_title(_('Item Details'))

        pannable = hildon.PannableArea()
        pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)

        item = self.item_window_header.item

        row = 0;
        table = gtk.Table()
        table.set_col_spacing(0, 12)
        self._add_to_table (table, secondary_color.to_string(), _('Title'), item.title, row)
        row += 1
        self._add_to_table (table, secondary_color.to_string(), _('Published'),
                            time.strftime('%x', item.updated_parsed), row)

        pannable.add_with_viewport(table)

        table.show_all()
        pannable.show()
        dialog.vbox.add(pannable)

        dialog.show()

    def _up_button_clicked(self, button):

        # http://faq.pygtk.org/index.py?req=show&file=faq13.051.htp
        path = self.model.get_path(self.model_iter)
        position = path[-1]
        if position == 0:
            return
        prev_path = list(path)[:-1]
        prev_path.append(position - 1)
        self.model_iter = self.model.get_iter(tuple(prev_path))

        if self.model_iter == None:
            return

        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
        self.set_item(item)

    def _down_button_clicked(self, button):

        if self.model_iter == None:
            return

        self.model_iter = self.model.iter_next(self.model_iter)

        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
        self.set_item(item)

    def set_item_from_model(self, model, path):

        self.model = model
        self.model_iter = model.get_iter(path)

        item_iter = model.get_iter(path)
        item = model.get_value(item_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)

        self.set_item(item)

    def set_item(self, item):

        author = get_author_from_item(item)
        if author == '':
            author = self.feed_data.feed.title
        self.set_title(author)

        # Set item in header
        self.item_window_header.set_item(item)

        # Get content type and body
        content_type = 'text/html'
        if 'summary' in item:
            if 'summary_detail' in item and 'type' in item.summary_detail:
                content_type = item.summary_detail.type
            body = item.summary
        elif 'content' in item:
            # We are only considering the first content. TODO check more?
            if 'type' in item.content[0]:
                content_type = item.content[0].type
            body = item.content[0].value
        else:
            # Should never happen
            body = _('No text')

        # Write HTML
        self.view.load_string(body.encode(self.feed_data.encoding), content_type, self.feed_data.encoding, '')

        # Update button sensitiviness
        toolbar_button_list = self.toolbar.get_children()
        path = self.model.get_path(self.model_iter)
        position = path[-1]
        if position == 0:
            self.item_window_header.button_up.set_sensitive(False)
            toolbar_button_list[0].set_sensitive(False)
        elif position == len(self.model) - 1:
            self.item_window_header.button_down.set_sensitive(False)
            toolbar_button_list[1].set_sensitive(False)
        else:
            self.item_window_header.button_up.set_sensitive(True)
            self.item_window_header.button_down.set_sensitive(True)
            toolbar_button_list[0].set_sensitive(True)
            toolbar_button_list[1].set_sensitive(True)

        # Mark item as read
        item['read'] = True

        # Emit item read
        self.emit('item-read', self.model.get_path(self.model_iter))

        # Force online/offline status evaluation
        self._on_connection_changed(self._conn_manager)

    def _download_item_cb(self, button):
        item = self.item_window_header.item
        url_file = item.enclosures[0]['href']
        file = os.path.basename(urllib.url2pathname(url_file))
        folder = self.settings.auto_download_folder

        hildon.hildon_gtk_window_set_progress_indicator(self, True)
        self.manager.download_item(url_file, folder+file, self._item_downloaded_cb)

    def _item_downloaded_cb(self, downloaded):
        if downloaded:
            message = _('Item downloaded')
        else:
            message = _('Error downloading item')
        hildon.hildon_banner_show_information(self, '', message)
        # Remove progress information
        hildon.hildon_gtk_window_set_progress_indicator(self, False)

class FindView(hildon.GtkTreeView):

    FIND_SITE_VIEW = 0

    def __init__(self, ui_mode):
        super(FindView, self).__init__(ui_mode)

        # Add columns
        text_renderer = gtk.CellRendererText()
        text_renderer.set_property('ellipsize', ELLIPSIZE_END)
        column = gtk.TreeViewColumn('Name', text_renderer, markup=self.FIND_SITE_VIEW)
        column.set_expand(True)
        self.append_column(column)


class FindWindow(hildon.StackableWindow):
    __gsignals__ = {
        "urls-found": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
        }

    FIND_MODEL_SITE_COLUMN = 0
    FIND_MODEL_URL_COLUMN = 1

    def __init__(self, feedsinfo, key_words, feedsWindow):
        super(FindWindow, self).__init__()

        self.feeds_info = feedsinfo

        self.view = FindView(gtk.HILDON_UI_MODE_EDIT)
        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.view.get_selection().unselect_all()

        self.edit_toolbar = hildon.EditToolbar()

        self.edit_toolbar_button_handler = 0
        self.edit_toolbar.set_label(_('Select Feeds to add'))
        self.edit_toolbar.set_button_label(_('Add'))
        self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)

        if self.edit_toolbar.handler_is_connected (self.edit_toolbar_button_handler):
            self.edit_toolbar.disconnect(self.edit_toolbar_button_handler)
        self.edit_toolbar_button_handler = \
            self.edit_toolbar.connect('button-clicked', self._add_button_clicked_cb, feedsWindow)

        self.set_edit_toolbar(self.edit_toolbar)
        self.edit_toolbar.show()
        self.fullscreen()

        self.view.set_model (gtk.ListStore(str, str))

        store = self.view.get_model()

        for feed_info in self.feeds_info:
            feed_iter = store.append()

            store.set(feed_iter,
                      self.FIND_MODEL_SITE_COLUMN, gobject.markup_escape_text(unescape(feed_info['sitename'])),
                      self.FIND_MODEL_URL_COLUMN, feed_info['dataurl'])

        self.view.show()

        self.align = gtk.Alignment(0, 0, 1, 1)
        self.align.set_padding(4, 0 , 16, 16)

        pannable = hildon.PannableArea()
        pannable.add (self.view)
        pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
        pannable.show()

        self.align.add(pannable)
        self.add(self.align)
        self.align.show()


    def _add_button_clicked_cb(self, button, feedsWindow):
        selection = self.view.get_selection()
        selected_rows = selection.get_selected_rows()
        model, paths = selected_rows
        if not paths:
            hildon.hildon_banner_show_information(self, '', _('Please select at least one Feed'))
            return

        urls = [model[path][self.FIND_MODEL_URL_COLUMN] for path in paths]

        self.emit('urls-found', urls)

        # restore normal mode
        self._restore_normal_mode(button)

    def _restore_normal_mode(self, button):
        self.destroy()
